
function [im_reco, e, V, M, D, energy]=riders(number)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%    function [im_reco, e, V, M, D, energy]=riders(number)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% number --> rider to show
%
% im_reco -->matrix with the reconstruction images
% e --> reconstruction error 
% V --> eigenvectors
% M --> average image
% D --> eigenvalues
% energy --> energy
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reading the original data
display=1; %Variable which controlls the visualization (1) or not (0) 
N=9; %number of examples (rider images)
Ny_original=254; 
Nx_original=273;
Ny=round(Ny_original/4);
Nx=round(Nx_original/4);
M=Nx*Ny; %dimension of each image
Nv=N-1; %number of vectors

x1=imread('images/rossi.jpg');
x2=imread('images/sete.jpg');
x3=imread('images/pedrosa.jpg');
x4=imread('images/jorgelorenzo.jpg');
x5=imread('images/capirossi.jpg');
x6=imread('images/biaggi.jpg');
x7=imread('images/elias.jpg');
x8=imread('images/rolfo.jpg');
x9=imread('images/poggiali.jpg');


x1=imresize(x1,[Nx Ny],'bilinear');
x2=imresize(x2,[Nx Ny],'bilinear');
x3=imresize(x3,[Nx Ny],'bilinear');
x4=imresize(x4,[Nx Ny],'bilinear');
x5=imresize(x5,[Nx Ny],'bilinear');
x6=imresize(x6,[Nx Ny],'bilinear');
x7=imresize(x7,[Nx Ny],'bilinear');
x8=imresize(x8,[Nx Ny],'bilinear');
x9=imresize(x9,[Nx Ny],'bilinear');

for i=1:N
comando=sprintf('x%d=[x%d(:,:,1) x%d(:,:,2) x%d(:,:,3)];',i,i,i,i);
eval(comando);
end

for i=1:N
comando=sprintf('x%d=x%d(:);',i,i);
eval(comando);
end

X=[x1 x2 x3 x4 x5 x6 x7 x8 x9]; %%each column is an example (a rider)
X=double(X);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% PCA transformation 
[V,D,M] = pc_evectors(X,Nv);

for i = 1:N
    Xm(:,i) = X(:,i) - M; 
   
end;

p=V'*Xm; 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Reconstruction 
num=0;
im_reco=[];
for num_vec=[1 2 3 4 5 6 7 8]  %number of vectors used durinth reconstruction

	num=num+D(num_vec);
	y=V(:,1:num_vec)*p(1:num_vec,:); %reconstruction
	ym=y+M*ones(1,N);
	energy(num_vec)=num/sum(D);  %energy  (maximo=1)
	im_reco=[im_reco ym(:,number)]; %reconstruction images
    
end

e=Inf*ones(1,Nv);
for i=1:Nv
    e(i)=norm(X(:,number)-im_reco(:,i)); % reconstruction error 
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Visualization 

if display==1
figure
paint_matrix(X',Nx,Ny)
title('Original Images')
figure
paint_matrix(im_reco',Nx,Ny)
title('Image Reconstruction')
figure
bar(e)
title('Error')
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Functions to visualizate data

function paint_matrix(matrix,x,y)
    [rows,cols]=size(matrix);
    
    for i=1:rows
        subplot(1,rows,i)
        imagen=uint8(matrix(i,:));
        paint(imagen,x,y)
    end
return
    

function paint(imagen,x,y)
image((reshape(imagen,x,y,3)));axis equal; axis off;
return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

